@@ -24,37 +24,16 @@
 #define __G_ASYNC_HELPER_H__
 
 #include <gio/gio.h>
-#include <glib-object.h>
 
 G_BEGIN_DECLS
 
-typedef struct
-{
-  gpointer       async_object;
-  GError *       error;
-  gpointer       user_data;
-} GAsyncResultData;
-
 typedef gboolean (*GFDSourceFunc) (gpointer     user_data,
 				   GIOCondition condition,
 				   int          fd);
-typedef gboolean (*GFDSourceObjectFunc) (GObject *object,
-					 GIOCondition condition,
-					 gpointer     user_data);
 
-void     _g_queue_async_result (GAsyncResultData *result,
-				gpointer         async_object,
-				GError          *error,
-				gpointer         user_data,
-				GSourceFunc      source_func);
-
-GSource *_g_fd_source_new_with_object (GObject      *object,
-				       int           fd,
-				       gushort       events,
-				       GCancellable *cancellable);
-GSource *_g_fd_source_new             (int           fd,
-				       gushort       events,
-				       GCancellable *cancellable);
+GSource *_g_fd_source_new      (int              fd,
+				gushort          events,
+				GCancellable    *cancellable);
 
 G_END_DECLS
 
