@@ -31,7 +31,6 @@
 #include "gioerror.h"
 #include "glibintl.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:gfileoutputstream
@@ -45,15 +44,14 @@
  * GFileOutputStream implements #GSeekable, which allows the output 
  * stream to jump to arbitrary positions in the file and to truncate
  * the file, provided the filesystem of the file supports these 
- * operations. In addition to the generic g_seekable_ API, 
- * GFileOutputStream has its own API for seeking and positioning. 
- * To find the position of a file output stream, use 
- * g_file_output_stream_tell(). To find out if a file output 
- * stream supports seeking, use g_file_output_stream_can_seek().
- * To position a file output stream, use g_file_output_stream_seek().
- * To find out if a file output stream supports truncating, use
- * g_file_output_stream_can_truncate(). To truncate a file output
- * stream, use g_file_output_stream_truncate().
+ * operations.
+ *
+ * To find the position of a file output stream, use g_seekable_tell().
+ * To find out if a file output stream supports seeking, use
+ * g_seekable_can_seek().To position a file output stream, use
+ * g_seekable_seek(). To find out if a file output stream supports
+ * truncating, use g_seekable_can_truncate(). To truncate a file output
+ * stream, use g_seekable_truncate().
  **/
 
 static void       g_file_output_stream_seekable_iface_init    (GSeekableIface       *iface);
@@ -70,7 +68,7 @@ static gboolean   g_file_output_stream_seekable_truncate      (GSeekable
 							       GCancellable         *cancellable,
 							       GError              **error);
 static void       g_file_output_stream_real_query_info_async  (GFileOutputStream    *stream,
-							       char                 *attributes,
+							       const char           *attributes,
 							       int                   io_priority,
 							       GCancellable         *cancellable,
 							       GAsyncReadyCallback   callback,
@@ -143,7 +141,7 @@ g_file_output_stream_init (GFileOutputStream *stream)
  **/
 GFileInfo *
 g_file_output_stream_query_info (GFileOutputStream      *stream,
-				    char                   *attributes,
+				    const char             *attributes,
 				    GCancellable           *cancellable,
 				    GError                **error)
 {
@@ -211,7 +209,7 @@ async_ready_callback_wrapper (GObject *source_object,
  **/
 void
 g_file_output_stream_query_info_async (GFileOutputStream     *stream,
-					  char                 *attributes,
+					  const char           *attributes,
 					  int                   io_priority,
 					  GCancellable         *cancellable,
 					  GAsyncReadyCallback   callback,
@@ -536,7 +534,7 @@ query_info_async_thread (GSimpleAsyncResult *res,
 
 static void
 g_file_output_stream_real_query_info_async (GFileOutputStream     *stream,
-					       char                 *attributes,
+					       const char           *attributes,
 					       int                   io_priority,
 					       GCancellable         *cancellable,
 					       GAsyncReadyCallback   callback,
@@ -571,6 +569,3 @@ g_file_output_stream_real_query_info_finish (GFileOutputStream     *stream,
   
   return NULL;
 }
-
-#define __G_FILE_OUTPUT_STREAM_C__
-#include "gioaliasdef.c"
