@@ -26,18 +26,24 @@
 #include "gsocketclient.h"
 
 #include <stdlib.h>
+#include <string.h>
 
 #include <gio/gioenumtypes.h>
 #include <gio/gsocketaddressenumerator.h>
 #include <gio/gsocketconnectable.h>
 #include <gio/gsocketconnection.h>
+#include <gio/gproxyaddressenumerator.h>
+#include <gio/gproxyaddress.h>
+#include <gio/gproxyconnection.h>
 #include <gio/gsimpleasyncresult.h>
 #include <gio/gcancellable.h>
 #include <gio/gioerror.h>
 #include <gio/gsocket.h>
 #include <gio/gnetworkaddress.h>
 #include <gio/gnetworkservice.h>
+#include <gio/gproxy.h>
 #include <gio/gsocketaddress.h>
+#include <gio/gtcpconnection.h>
 #include "glibintl.h"
 
 
@@ -70,7 +76,9 @@ enum
   PROP_FAMILY,
   PROP_TYPE,
   PROP_PROTOCOL,
-  PROP_LOCAL_ADDRESS
+  PROP_LOCAL_ADDRESS,
+  PROP_TIMEOUT,
+  PROP_ENABLE_PROXY,
 };
 
 struct _GSocketClientPrivate
@@ -79,6 +87,9 @@ struct _GSocketClientPrivate
   GSocketType type;
   GSocketProtocol protocol;
   GSocketAddress *local_address;
+  guint timeout;
+  gboolean enable_proxy;
+  GHashTable *app_proxies;
 };
 
 static GSocket *
@@ -115,9 +126,21 @@ create_socket (GSocketClient  *client,
 	}
     }
 
+  if (client->priv->timeout)
+    g_socket_set_timeout (socket, client->priv->timeout);
+
   return socket;
 }
 
+gboolean
+can_use_proxy (GSocketClient *client)
+{
+  GSocketClientPrivate *priv = client->priv;
+
+  return priv->enable_proxy
+          && priv->type == G_SOCKET_TYPE_STREAM;
+}
+
 static void
 g_socket_client_init (GSocketClient *client)
 {
@@ -125,6 +148,10 @@ g_socket_client_init (GSocketClient *client)
 					      G_TYPE_SOCKET_CLIENT,
 					      GSocketClientPrivate);
   client->priv->type = G_SOCKET_TYPE_STREAM;
+  client->priv->app_proxies = g_hash_table_new_full (g_str_hash,
+						     g_str_equal,
+						     g_free,
+						     NULL);
 }
 
 /**
@@ -153,6 +180,8 @@ g_socket_client_finalize (GObject *object)
 
   if (G_OBJECT_CLASS (g_socket_client_parent_class)->finalize)
     (*G_OBJECT_CLASS (g_socket_client_parent_class)->finalize) (object);
+
+  g_hash_table_unref (client->priv->app_proxies);
 }
 
 static void
@@ -181,6 +210,14 @@ g_socket_client_get_property (GObject    *object,
 	g_value_set_object (value, client->priv->local_address);
 	break;
 
+      case PROP_TIMEOUT:
+	g_value_set_uint (value, client->priv->timeout);
+	break;
+
+      case PROP_ENABLE_PROXY:
+	g_value_set_boolean (value, client->priv->enable_proxy);
+	break;
+
       default:
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -212,6 +249,14 @@ g_socket_client_set_property (GObject      *object,
       g_socket_client_set_local_address (client, g_value_get_object (value));
       break;
 
+    case PROP_TIMEOUT:
+      g_socket_client_set_timeout (client, g_value_get_uint (value));
+      break;
+
+    case PROP_ENABLE_PROXY:
+      g_socket_client_set_enable_proxy (client, g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -386,7 +431,7 @@ g_socket_client_set_local_address (GSocketClient  *client,
 				   GSocketAddress *address)
 {
   if (address)
-  g_object_ref (address);
+    g_object_ref (address);
 
   if (client->priv->local_address)
     {
@@ -396,6 +441,90 @@ g_socket_client_set_local_address (GSocketClient  *client,
   g_object_notify (G_OBJECT (client), "local-address");
 }
 
+/**
+ * g_socket_client_get_timeout:
+ * @client: a #GSocketClient
+ *
+ * Gets the I/O timeout time for sockets created by @client.
+ *
+ * See g_socket_client_set_timeout() for details.
+ *
+ * Returns: the timeout in seconds
+ *
+ * Since: 2.26
+ */
+guint
+g_socket_client_get_timeout (GSocketClient *client)
+{
+  return client->priv->timeout;
+}
+
+
+/**
+ * g_socket_client_set_timeout:
+ * @client: a #GSocketClient.
+ * @timeout: the timeout
+ *
+ * Sets the I/O timeout for sockets created by @client. @timeout is a
+ * time in seconds, or 0 for no timeout (the default).
+ *
+ * The timeout value affects the initial connection attempt as well,
+ * so setting this may cause calls to g_socket_client_connect(), etc,
+ * to fail with %G_IO_ERROR_TIMED_OUT.
+ *
+ * Since: 2.26
+ */
+void
+g_socket_client_set_timeout (GSocketClient *client,
+			     guint          timeout)
+{
+  if (client->priv->timeout == timeout)
+    return;
+
+  client->priv->timeout = timeout;
+  g_object_notify (G_OBJECT (client), "timeout");
+}
+
+/**
+ * g_socket_client_get_enable_proxy:
+ * @client: a #GSocketClient.
+ *
+ * Gets the proxy enable state; see g_socket_client_set_enable_proxy()
+ *
+ * Returns: whether proxying is enabled
+ *
+ * Since: 2.26
+ */
+gboolean
+g_socket_client_get_enable_proxy (GSocketClient *client)
+{
+  return client->priv->enable_proxy;
+}
+
+/**
+ * g_socket_client_set_enable_proxy:
+ * @client: a #GSocketClient.
+ * @enable: whether to enable proxies
+ *
+ * Sets whether or not @client attempts to make connections via a
+ * proxy server. When enabled (the default), #GSocketClient will use a
+ * #GProxyResolver to determine if a proxy protocol such as SOCKS is
+ * needed, and automatically do the necessary proxy negotiation.
+ *
+ * Since: 2.26
+ */
+void
+g_socket_client_set_enable_proxy (GSocketClient *client,
+				  gboolean       enable)
+{
+  enable = !!enable;
+  if (client->priv->enable_proxy == enable)
+    return;
+
+  client->priv->enable_proxy = enable;
+  g_object_notify (G_OBJECT (client), "enable-proxy");
+}
+
 static void
 g_socket_client_class_init (GSocketClientClass *class)
 {
@@ -445,6 +574,25 @@ g_socket_client_class_init (GSocketClientClass *class)
 							G_PARAM_CONSTRUCT |
                                                         G_PARAM_READWRITE |
                                                         G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class, PROP_TIMEOUT,
+				   g_param_spec_uint ("timeout",
+						      P_("Socket timeout"),
+						      P_("The I/O timeout for sockets, or 0 for none"),
+						      0, G_MAXUINT, 0,
+						      G_PARAM_CONSTRUCT |
+                                                      G_PARAM_READWRITE |
+                                                      G_PARAM_STATIC_STRINGS));
+
+   g_object_class_install_property (gobject_class, PROP_ENABLE_PROXY,
+				    g_param_spec_boolean ("enable-proxy",
+							  P_("Enable proxy"),
+							  P_("Enable proxy support"),
+							  TRUE,
+							  G_PARAM_CONSTRUCT |
+							  G_PARAM_READWRITE |
+							  G_PARAM_STATIC_STRINGS));
+
 }
 
 /**
@@ -484,14 +632,19 @@ g_socket_client_connect (GSocketClient       *client,
 			 GError             **error)
 {
   GSocketConnection *connection = NULL;
-  GSocketAddressEnumerator *enumerator;
+  GSocketAddressEnumerator *enumerator = NULL;
   GError *last_error, *tmp_error;
 
   last_error = NULL;
-  enumerator = g_socket_connectable_enumerate (connectable);
+
+  if (can_use_proxy (client))
+    enumerator = g_socket_connectable_proxy_enumerate (connectable);
+  else
+    enumerator = g_socket_connectable_enumerate (connectable);
+
   while (connection == NULL)
     {
-      GSocketAddress *address;
+      GSocketAddress *address = NULL;
       GSocket *socket;
 
       if (g_cancellable_is_cancelled (cancellable))
@@ -503,7 +656,8 @@ g_socket_client_connect (GSocketClient       *client,
 
       tmp_error = NULL;
       address = g_socket_address_enumerator_next (enumerator, cancellable,
-						  &tmp_error);
+	      					  &tmp_error);
+
       if (address == NULL)
 	{
 	  if (tmp_error)
@@ -533,6 +687,72 @@ g_socket_client_connect (GSocketClient       *client,
 	  g_object_unref (socket);
 	}
 
+      if (connection &&
+	  G_IS_PROXY_ADDRESS (address) &&
+	  client->priv->enable_proxy)
+	{
+	  GProxyAddress *proxy_addr = G_PROXY_ADDRESS (address);
+	  const gchar *protocol;
+	  GProxy *proxy;
+
+	  protocol = g_proxy_address_get_protocol (proxy_addr);
+	  proxy = g_proxy_get_default_for_protocol (protocol);
+
+          /* The connection should not be anything else then TCP Connection,
+           * but let's put a safety guard in case
+	   */
+          if (!G_IS_TCP_CONNECTION (connection))
+            {
+              g_critical ("Trying to proxy over non-TCP connection, this is "
+                          "most likely a bug in GLib IO library.");
+
+              g_set_error_literal (&last_error,
+                  G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+                  _("Trying to proxy over non-TCP connection is not supported."));
+
+	      g_object_unref (connection);
+	      connection = NULL;
+            }
+          else if (proxy)
+	    {
+              GIOStream *io_stream;
+              GTcpConnection *old_connection = G_TCP_CONNECTION (connection);
+
+	      io_stream = g_proxy_connect (proxy,
+					   G_IO_STREAM (old_connection),
+					   proxy_addr,
+					   cancellable,
+					   &last_error);
+
+              if (io_stream)
+                {
+                  if (G_IS_SOCKET_CONNECTION (io_stream))
+                    connection = G_SOCKET_CONNECTION (g_object_ref (io_stream));
+                  else
+                    connection = _g_proxy_connection_new (old_connection,
+                                                          io_stream);
+
+                  g_object_unref (io_stream);
+                }
+              else
+                {
+                  connection = NULL;
+                }
+
+              g_object_unref (old_connection);
+	      g_object_unref (proxy);
+	    }
+	  else if (!g_hash_table_lookup_extended (client->priv->app_proxies,
+						  protocol, NULL, NULL))
+	    {
+	      g_set_error (&last_error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+			   _("Proxy protocol '%s' is not supported."),
+			   protocol);
+	      g_object_unref (connection);
+	      connection = NULL;
+	    }
+	}
+
       g_object_unref (address);
     }
   g_object_unref (enumerator);
@@ -542,7 +762,7 @@ g_socket_client_connect (GSocketClient       *client,
 
 /**
  * g_socket_client_connect_to_host:
- * @client: a #SocketClient
+ * @client: a #GSocketClient
  * @host_and_port: the name and optionally port of the host to connect to
  * @default_port: the default port to connect to
  * @cancellable: a #GCancellable, or %NULL
@@ -646,6 +866,59 @@ g_socket_client_connect_to_service (GSocketClient  *client,
   return connection;
 }
 
+/**
+ * g_socket_client_connect_to_uri:
+ * @client: a #GSocketClient
+ * @uri: A network URI
+ * @default_port: the default port to connect to
+ * @cancellable: a #GCancellable, or %NULL
+ * @error: a pointer to a #GError, or %NULL
+ *
+ * This is a helper function for g_socket_client_connect().
+ *
+ * Attempts to create a TCP connection with a network URI.
+ *
+ * @uri may be any valid URI containing an "authority" (hostname/port)
+ * component. If a port is not specified in the URI, @default_port
+ * will be used.
+ *
+ * Using this rather than g_socket_client_connect() or
+ * g_socket_client_connect_to_host() allows #GSocketClient to
+ * determine when to use application-specific proxy protocols.
+ *
+ * Upon a successful connection, a new #GSocketConnection is constructed
+ * and returned.  The caller owns this new object and must drop their
+ * reference to it when finished with it.
+ *
+ * In the event of any failure (DNS error, service not found, no hosts
+ * connectable) %NULL is returned and @error (if non-%NULL) is set
+ * accordingly.
+ *
+ * Returns: a #GSocketConnection on success, %NULL on error.
+ *
+ * Since: 2.26
+ */
+GSocketConnection *
+g_socket_client_connect_to_uri (GSocketClient  *client,
+				const gchar    *uri,
+				guint16         default_port,
+				GCancellable   *cancellable,
+				GError        **error)
+{
+  GSocketConnectable *connectable;
+  GSocketConnection *connection;
+
+  connectable = g_network_address_parse_uri (uri, default_port, error);
+  if (connectable == NULL)
+    return NULL;
+
+  connection = g_socket_client_connect (client, connectable,
+					cancellable, error);
+  g_object_unref (connectable);
+
+  return connection;
+}
+
 typedef struct
 {
   GSimpleAsyncResult *result;
@@ -653,7 +926,9 @@ typedef struct
   GSocketClient *client;
 
   GSocketAddressEnumerator *enumerator;
+  GProxyAddress *proxy_addr;
   GSocket *current_socket;
+  GSocketConnection *connection;
 
   GError *last_error;
 } GSocketClientAsyncConnectData;
@@ -661,8 +936,6 @@ typedef struct
 static void
 g_socket_client_async_connect_complete (GSocketClientAsyncConnectData *data)
 {
-  GSocketConnection *connection;
-
   if (data->last_error)
     {
       g_simple_async_result_set_from_error (data->result, data->last_error);
@@ -670,20 +943,22 @@ g_socket_client_async_connect_complete (GSocketClientAsyncConnectData *data)
     }
   else
     {
-      g_assert (data->current_socket);
+      g_assert (data->connection);
 
-      g_socket_set_blocking (data->current_socket, TRUE);
-
-      connection = g_socket_connection_factory_create_connection (data->current_socket);
-      g_object_unref (data->current_socket);
       g_simple_async_result_set_op_res_gpointer (data->result,
-						 connection,
+						 data->connection,
 						 g_object_unref);
     }
 
   g_simple_async_result_complete (data->result);
   g_object_unref (data->result);
   g_object_unref (data->enumerator);
+  if (data->cancellable)
+    g_object_unref (data->cancellable);
+  if (data->current_socket)
+    g_object_unref (data->current_socket);
+  if (data->proxy_addr)
+    g_object_unref (data->proxy_addr);
   g_slice_free (GSocketClientAsyncConnectData, data);
 }
 
@@ -701,6 +976,98 @@ set_last_error (GSocketClientAsyncConnectData *data,
   data->last_error = error;
 }
 
+static void
+enumerator_next_async (GSocketClientAsyncConnectData *data)
+{
+  g_socket_address_enumerator_next_async (data->enumerator,
+					  data->cancellable,
+					  g_socket_client_enumerator_callback,
+					  data);
+}
+
+static void
+g_socket_client_proxy_connect_callback (GObject      *object,
+					GAsyncResult *result,
+					gpointer      user_data)
+{
+  GSocketClientAsyncConnectData *data = user_data;
+  GIOStream *io_stream;
+  GTcpConnection *old_connection = G_TCP_CONNECTION (data->connection);
+
+  io_stream = g_proxy_connect_finish (G_PROXY (object),
+				      result,
+				      &data->last_error);
+
+  if (io_stream)
+    {
+      if (G_IS_SOCKET_CONNECTION (io_stream))
+        data->connection = G_SOCKET_CONNECTION (g_object_ref (io_stream));
+      else
+        data->connection = _g_proxy_connection_new (old_connection,
+                                                    io_stream);
+      g_object_unref (io_stream);
+    }
+  else
+    {
+      data->connection = NULL;
+    }
+
+  g_object_unref (old_connection);
+
+  g_socket_client_async_connect_complete (data);
+}
+
+static void
+g_socket_client_proxy_connect (GSocketClientAsyncConnectData *data)
+{
+  GProxy *proxy;
+  const gchar *protocol = g_proxy_address_get_protocol (data->proxy_addr);
+
+  proxy = g_proxy_get_default_for_protocol (protocol);
+
+  /* The connection should not be anything else then TCP Connection,
+   * but let's put a safety guard in case
+   */
+  if (!G_IS_TCP_CONNECTION (data->connection))
+    {
+      g_critical ("Trying to proxy over non-TCP connection, this is "
+          "most likely a bug in GLib IO library.");
+
+      g_set_error_literal (&data->last_error,
+          G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+          _("Trying to proxy over non-TCP connection is not supported."));
+
+      g_object_unref (data->connection);
+      data->connection = NULL;
+
+      enumerator_next_async (data);
+    }
+  else if (proxy)
+    {
+      g_proxy_connect_async (proxy,
+                             G_IO_STREAM (data->connection),
+                             data->proxy_addr,
+                             data->cancellable,
+                             g_socket_client_proxy_connect_callback,
+                             data);
+      g_object_unref (proxy);
+    }
+  else if (!g_hash_table_lookup_extended (data->client->priv->app_proxies,
+					  protocol, NULL, NULL))
+    {
+      g_clear_error (&data->last_error);
+
+      g_set_error (&data->last_error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+          _("Proxy protocol '%s' is not supported."),
+          protocol);
+
+      g_object_unref (data->connection);
+      data->connection = NULL;
+
+      enumerator_next_async (data);
+    }
+}
+
 static gboolean
 g_socket_client_socket_callback (GSocket *socket,
 				 GIOCondition condition,
@@ -716,6 +1083,9 @@ g_socket_client_socket_callback (GSocket *socket,
       data->current_socket = NULL;
       g_cancellable_set_error_if_cancelled (data->cancellable,
 					    &data->last_error);
+
+      g_socket_client_async_connect_complete (data);
+      return FALSE;
     }
   else
     {
@@ -723,18 +1093,27 @@ g_socket_client_socket_callback (GSocket *socket,
       if (!g_socket_check_connect_result (data->current_socket, &error))
 	{
 	  set_last_error (data, error);
+	  g_object_unref (data->current_socket);
+	  data->current_socket = NULL;
 
 	  /* try next one */
-	  g_socket_address_enumerator_next_async (data->enumerator,
-						  data->cancellable,
-						  g_socket_client_enumerator_callback,
-						  data);
+	  enumerator_next_async (data);
 
 	  return FALSE;
 	}
     }
 
-  g_socket_client_async_connect_complete (data);
+  g_socket_set_blocking (data->current_socket, TRUE);
+
+  data->connection =
+    g_socket_connection_factory_create_connection (data->current_socket);
+  g_object_unref (data->current_socket);
+  data->current_socket = NULL;
+
+  if (data->proxy_addr)
+    g_socket_client_proxy_connect (data);
+  else
+    g_socket_client_async_connect_complete (data);
 
   return FALSE;
 }
@@ -745,7 +1124,7 @@ g_socket_client_enumerator_callback (GObject      *object,
 				     gpointer      user_data)
 {
   GSocketClientAsyncConnectData *data = user_data;
-  GSocketAddress *address;
+  GSocketAddress *address = NULL;
   GSocket *socket;
   GError *tmp_error = NULL;
 
@@ -772,6 +1151,10 @@ g_socket_client_enumerator_callback (GObject      *object,
       return;
     }
 
+  if (G_IS_PROXY_ADDRESS (address) &&
+      data->client->priv->enable_proxy)
+    data->proxy_addr = g_object_ref (G_PROXY_ADDRESS (address));
+
   g_clear_error (&data->last_error);
 
   socket = create_socket (data->client, address, &data->last_error);
@@ -809,13 +1192,10 @@ g_socket_client_enumerator_callback (GObject      *object,
 	  data->last_error = tmp_error;
 	  g_object_unref (socket);
 	}
-      g_object_unref (address);
     }
 
-  g_socket_address_enumerator_next_async (data->enumerator,
-					  data->cancellable,
-					  g_socket_client_enumerator_callback,
-					  data);
+  g_object_unref (address);
+  enumerator_next_async (data);
 }
 
 /**
@@ -845,7 +1225,7 @@ g_socket_client_connect_async (GSocketClient       *client,
 
   g_return_if_fail (G_IS_SOCKET_CLIENT (client));
 
-  data = g_slice_new (GSocketClientAsyncConnectData);
+  data = g_slice_new0 (GSocketClientAsyncConnectData);
 
   data->result = g_simple_async_result_new (G_OBJECT (client),
 					    callback, user_data,
@@ -853,14 +1233,13 @@ g_socket_client_connect_async (GSocketClient       *client,
   data->client = client;
   if (cancellable)
     data->cancellable = g_object_ref (cancellable);
-  else
-    data->cancellable = NULL;
-  data->last_error = NULL;
-  data->enumerator = g_socket_connectable_enumerate (connectable);
 
-  g_socket_address_enumerator_next_async (data->enumerator, cancellable,
-					  g_socket_client_enumerator_callback,
-					  data);
+  if (can_use_proxy (client))
+      data->enumerator = g_socket_connectable_proxy_enumerate (connectable);
+  else
+      data->enumerator = g_socket_connectable_enumerate (connectable);
+
+  enumerator_next_async (data);
 }
 
 /**
@@ -940,6 +1319,52 @@ g_socket_client_connect_to_service_async (GSocketClient       *client,
   g_object_unref (connectable);
 }
 
+/**
+ * g_socket_client_connect_to_uri_async:
+ * @client: a #GSocketClient
+ * @uri: a network uri
+ * @default_port: the default port to connect to
+ * @cancellable: a #GCancellable, or %NULL
+ * @callback: a #GAsyncReadyCallback
+ * @user_data: user data for the callback
+ *
+ * This is the asynchronous version of g_socket_client_connect_to_uri().
+ *
+ * When the operation is finished @callback will be
+ * called. You can then call g_socket_client_connect_to_uri_finish() to get
+ * the result of the operation.
+ *
+ * Since: 2.26
+ */
+void
+g_socket_client_connect_to_uri_async (GSocketClient        *client,
+				      const gchar          *uri,
+				      guint16               default_port,
+				      GCancellable         *cancellable,
+				      GAsyncReadyCallback   callback,
+				      gpointer              user_data)
+{
+  GSocketConnectable *connectable;
+  GError *error;
+
+  error = NULL;
+  connectable = g_network_address_parse_uri (uri, default_port, &error);
+  if (connectable == NULL)
+    {
+      g_simple_async_report_gerror_in_idle (G_OBJECT (client),
+					    callback, user_data, error);
+      g_error_free (error);
+    }
+  else
+    {
+      g_socket_client_connect_async (client,
+				     connectable, cancellable,
+				     callback, user_data);
+      g_object_unref (connectable);
+    }
+}
+
+
 /**
  * g_socket_client_connect_finish:
  * @client: a #GSocketClient.
@@ -1007,3 +1432,52 @@ g_socket_client_connect_to_service_finish (GSocketClient  *client,
 {
   return g_socket_client_connect_finish (client, result, error);
 }
+
+/**
+ * g_socket_client_connect_to_uri_finish:
+ * @client: a #GSocketClient.
+ * @result: a #GAsyncResult.
+ * @error: a #GError location to store the error occuring, or %NULL to
+ * ignore.
+ *
+ * Finishes an async connect operation. See g_socket_client_connect_to_uri_async()
+ *
+ * Returns: a #GSocketConnection on success, %NULL on error.
+ *
+ * Since: 2.26
+ */
+GSocketConnection *
+g_socket_client_connect_to_uri_finish (GSocketClient  *client,
+				       GAsyncResult   *result,
+				       GError        **error)
+{
+  return g_socket_client_connect_finish (client, result, error);
+}
+
+/**
+ * g_socket_client_add_application_proxy:
+ * @client: a #GSocketClient
+ * @protocol: The proxy protocol
+ *
+ * Enable proxy protocols to be handled by the application. When the
+ * indicated proxy protocol is returned by the #GProxyResolver,
+ * #GSocketClient will consider this protocol as supported but will
+ * not try find a #GProxy instance to handle handshaking. The
+ * application must check for this case by calling
+ * g_socket_connection_get_remote_address() on the returned
+ * #GSocketConnection, and seeing if it's a #GProxyAddress of the
+ * appropriate type, to determine whether or not it needs to handle
+ * the proxy handshaking itself.
+ *
+ * This should be used for proxy protocols that are dialects of
+ * another protocol such as HTTP proxy. It also allows cohabitation of
+ * proxy protocols that are reused between protocols. A good example
+ * is HTTP. It can be used to proxy HTTP, FTP and Gopher and can also
+ * be use as generic socket proxy through the HTTP CONNECT method.
+ */
+void
+g_socket_client_add_application_proxy (GSocketClient *client,
+			               const gchar   *protocol)
+{
+  g_hash_table_insert (client->priv->app_proxies, g_strdup (protocol), NULL);
+}
