@@ -46,7 +46,6 @@
 #include <windows.h>
 #endif
 
-#include "galias.h"
 
 GDate*
 g_date_new (void)
@@ -928,7 +927,7 @@ g_date_set_time_t (GDate *date,
  * Sets the value of a date from a #GTime value.
  * The time to date conversion is done using the user's current timezone.
  *
- * @Deprecated:2.10: Use g_date_set_time_t() instead.
+ * Deprecated: 2.10: Use g_date_set_time_t() instead.
  */
 void
 g_date_set_time (GDate *date,
@@ -946,6 +945,8 @@ g_date_set_time (GDate *date,
  * @tv_usec member is ignored, because #GDate can't make use of the
  * additional precision.
  *
+ * The time to date conversion is done using the user's current timezone.
+ *
  * Since: 2.10
  */
 void
@@ -1899,7 +1900,3 @@ g_date_strftime (gchar       *s,
   return retval;
 #endif
 }
-
-#define __G_DATE_C__
-#include "galiasdef.c"
-
