@@ -43,6 +43,7 @@
 #include <glib/gconvert.h>
 #include <glib/gdataset.h>
 #include <glib/gdate.h>
+#include <glib/gdatetime.h>
 #include <glib/gdir.h>
 #include <glib/gerror.h>
 #include <glib/gfileutils.h>
@@ -87,6 +88,7 @@
 #include <glib/gurifuncs.h>
 #include <glib/gutils.h>
 #include <glib/gvarianttype.h>
+#include <glib/gvariant.h>
 #ifdef G_PLATFORM_WIN32
 #include <glib/gwin32.h>
 #endif
