@@ -45,7 +45,6 @@
 #include "gdebug.h"
 #include "gprintfint.h"
 #include "gthreadprivate.h"
-#include "galias.h"
 
 #ifdef G_OS_WIN32
 #include <process.h>		/* For getpid() */
@@ -575,12 +574,6 @@ g_return_if_fail_warning (const char *log_domain,
 			  const char *pretty_function,
 			  const char *expression)
 {
-  /*
-   * Omit the prefix used by the PLT-reduction
-   * technique used in GTK+. 
-   */
-  if (g_str_has_prefix (pretty_function, "IA__"))
-    pretty_function += 4;
   g_log (log_domain,
 	 G_LOG_LEVEL_CRITICAL,
 	 "%s: assertion `%s' failed",
@@ -616,12 +609,6 @@ g_assert_warning (const char *log_domain,
 		  const char *pretty_function,
 		  const char *expression)
 {
-  /*
-   * Omit the prefix used by the PLT-reduction
-   * technique used in GTK+. 
-   */
-  if (g_str_has_prefix (pretty_function, "IA__"))
-    pretty_function += 4;
   g_log (log_domain,
 	 G_LOG_LEVEL_ERROR,
 	 expression 
@@ -1161,6 +1148,3 @@ _g_debug_init (void)
       g_log_set_always_fatal (fatal_mask);
     }
 }
-
-#define __G_MESSAGES_C__
-#include "galiasdef.c"
