@@ -31,7 +31,6 @@
 #include "gmirroringtable.h"
 #include "gscripttable.h"
 #include "gunicodeprivate.h"
-#include "galias.h"
 
 #define ATTR_TABLE(Page) (((Page) <= G_UNICODE_LAST_PAGE_PART1) \
                           ? attr_table_part1[Page] \
@@ -479,6 +478,10 @@ g_unichar_iswide (gunichar c)
  * the converse is not necessarily true.  See the
  * <ulink url="http://www.unicode.org/reports/tr11/">Unicode Standard
  * Annex #11</ulink> for details.
+ *
+ * If a character passes the g_unichar_iswide() test then it will also pass
+ * this test, but not the other way around.  Note that some characters may
+ * pas both this test and g_unichar_iszerowidth().
  * 
  * Return value: %TRUE if the character is wide in legacy East Asian locales
  *
@@ -1292,7 +1295,3 @@ g_unichar_get_script (gunichar ch)
   else 
     return g_unichar_get_script_bsearch (ch); 
 }
-
-
-#define __G_UNIPROP_C__
-#include "galiasdef.c"
