@@ -31,7 +31,6 @@
 
 #include <string.h>
 
-#include "galias.h"
 
 /* GVariantSerialiser
  *
@@ -139,6 +138,34 @@ g_variant_serialised_check (GVariantSerialised serialised)
   else
     g_assert (serialised.size == 0 || serialised.data != NULL);
 
+  /* Depending on the native alignment requirements of the machine, the
+   * compiler will insert either 3 or 7 padding bytes after the char.
+   * This will result in the sizeof() the struct being 12 or 16.
+   * Subtract 9 to get 3 or 7 which is a nice bitmask to apply to get
+   * the alignment bits that we "care about" being zero: in the
+   * 4-aligned case, we care about 2 bits, and in the 8-aligned case, we
+   * care about 3 bits.
+   */
+  alignment &= sizeof (struct {
+                         char a;
+                         union {
+                           guint64 x;
+                           void *y;
+                           gdouble z;
+                         } b;
+                       }
+                      ) - 9;
+
+  /* Some OSes (FreeBSD is a known example) have a malloc() that returns
+   * unaligned memory if you request small sizes.  'malloc (1);', for
+   * example, has been seen to return pointers aligned to 6 mod 16.
+   *
+   * Check if this is a small allocation and return without enforcing
+   * the alignment assertion if this is the case.
+   */
+  if (serialised.size <= alignment)
+    return;
+
   g_assert_cmpint (alignment & (gsize) serialised.data, ==, 0);
 }
 
@@ -333,7 +360,7 @@ gvs_variable_sized_maybe_needed_size (GVariantTypeInfo         *type_info,
 {
   if (n_children)
     {
-      GVariantSerialised child = {  };
+      GVariantSerialised child = { 0, };
 
       gvs_filler (&child, children[0]);
 
@@ -414,7 +441,7 @@ static GVariantSerialised
 gvs_fixed_sized_array_get_child (GVariantSerialised value,
                                  gsize              index_)
 {
-  GVariantSerialised child = {  };
+  GVariantSerialised child = { 0, };
 
   child.type_info = g_variant_type_info_element (value.type_info);
   g_variant_type_info_query (child.type_info, NULL, &child.size);
@@ -443,7 +470,7 @@ gvs_fixed_sized_array_serialise (GVariantSerialised        value,
                                  const gpointer           *children,
                                  gsize                     n_children)
 {
-  GVariantSerialised child = {  };
+  GVariantSerialised child = { 0, };
   gsize i;
 
   child.type_info = g_variant_type_info_element (value.type_info);
@@ -460,7 +487,7 @@ gvs_fixed_sized_array_serialise (GVariantSerialised        value,
 static gboolean
 gvs_fixed_sized_array_is_normal (GVariantSerialised value)
 {
-  GVariantSerialised child = {  };
+  GVariantSerialised child = { 0, };
 
   child.type_info = g_variant_type_info_element (value.type_info);
   g_variant_type_info_query (child.type_info, NULL, &child.size);
@@ -620,7 +647,7 @@ static GVariantSerialised
 gvs_variable_sized_array_get_child (GVariantSerialised value,
                                     gsize              index_)
 {
-  GVariantSerialised child = {  };
+  GVariantSerialised child = { 0, };
   gsize offset_size;
   gsize last_end;
   gsize start;
@@ -676,7 +703,7 @@ gvs_variable_sized_array_needed_size (GVariantTypeInfo         *type_info,
 
   for (i = 0; i < n_children; i++)
     {
-      GVariantSerialised child = {  };
+      GVariantSerialised child = { 0, };
 
       offset += (-offset) & alignment;
       gvs_filler (&child, children[i]);
@@ -706,7 +733,7 @@ gvs_variable_sized_array_serialise (GVariantSerialised        value,
 
   for (i = 0; i < n_children; i++)
     {
-      GVariantSerialised child = {  };
+      GVariantSerialised child = { 0, };
 
       while (offset & alignment)
         value.data[offset++] = '\0';
@@ -723,7 +750,7 @@ gvs_variable_sized_array_serialise (GVariantSerialised        value,
 static gboolean
 gvs_variable_sized_array_is_normal (GVariantSerialised value)
 {
-  GVariantSerialised child = {  };
+  GVariantSerialised child = { 0, };
   gsize offsets_array_size;
   guchar *offsets_array;
   guint offset_size;
@@ -825,7 +852,7 @@ gvs_tuple_get_child (GVariantSerialised value,
                      gsize              index_)
 {
   const GVariantMemberInfo *member_info;
-  GVariantSerialised child = {  };
+  GVariantSerialised child = { 0, };
   gsize offset_size;
   gsize start, end;
 
@@ -936,7 +963,7 @@ gvs_tuple_needed_size (GVariantTypeInfo         *type_info,
         offset += fixed_size;
       else
         {
-          GVariantSerialised child = {  };
+          GVariantSerialised child = { 0, };
 
           gvs_filler (&child, children[i]);
           offset += child.size;
@@ -962,7 +989,7 @@ gvs_tuple_serialise (GVariantSerialised        value,
   for (i = 0; i < n_children; i++)
     {
       const GVariantMemberInfo *member_info;
-      GVariantSerialised child = {  };
+      GVariantSerialised child = { 0, };
       guint alignment;
 
       member_info = g_variant_type_info_member_info (value.type_info, i);
@@ -1041,6 +1068,9 @@ gvs_tuple_is_normal (GVariantSerialised value)
 
           end = gvs_read_unaligned_le (value.data + offset_ptr, offset_size);
           break;
+
+        default:
+          g_assert_not_reached ();
         }
 
       if (end < offset || end > offset_ptr)
@@ -1109,7 +1139,7 @@ static inline GVariantSerialised
 gvs_variant_get_child (GVariantSerialised value,
                        gsize              index_)
 {
-  GVariantSerialised child = {  };
+  GVariantSerialised child = { 0, };
 
   /* NOTE: not O(1) and impossible for it to be... */
   if (value.size)
@@ -1166,7 +1196,7 @@ gvs_variant_needed_size (GVariantTypeInfo         *type_info,
                          const gpointer           *children,
                          gsize                     n_children)
 {
-  GVariantSerialised child = {  };
+  GVariantSerialised child = { 0, };
   const gchar *type_string;
 
   gvs_filler (&child, children[0]);
@@ -1181,7 +1211,7 @@ gvs_variant_serialise (GVariantSerialised        value,
                        const gpointer           *children,
                        gsize                     n_children)
 {
-  GVariantSerialised child = {  };
+  GVariantSerialised child = { 0, };
   const gchar *type_string;
 
   child.data = value.data;
@@ -1558,15 +1588,11 @@ gboolean
 g_variant_serialiser_is_string (gconstpointer data,
                                 gsize         size)
 {
-  const gchar *string = data;
+  const gchar *end;
 
-  if (size == 0)
-    return FALSE;
+  g_utf8_validate (data, size, &end);
 
-  if (string[size - 1] != '\0')
-    return FALSE;
-
-  return strlen (string) == size - 1;
+  return data == end - (size - 1);
 }
 
 /* < private >
@@ -1655,7 +1681,4 @@ g_variant_serialiser_is_signature (gconstpointer data,
 }
 
 /* Epilogue {{{1 */
-#define __G_VARIANT_SERIALISER_C__
-#include "galiasdef.c"
-
 /* vim:set foldmethod=marker: */
