@@ -28,7 +28,6 @@
 #include "gparam.h"
 #include "gparamspecs.h"
 #include "gvaluecollector.h"
-#include "gobjectalias.h"
 
 
 /**
@@ -527,7 +526,7 @@ g_param_spec_set_qdata_full (GParamSpec    *pspec,
  * @quark: a #GQuark, naming the user data pointer
  *
  * Gets back user data pointers stored via g_param_spec_set_qdata()
- * and removes the @data from @pspec without invoking it's destroy()
+ * and removes the @data from @pspec without invoking its destroy()
  * function (if any was set).  Usually, calling this function is only
  * required to update user data pointers with a destroy notifier.
  *
@@ -1496,6 +1495,3 @@ g_value_dup_param (const GValue *value)
 
   return value->data[0].v_pointer ? g_param_spec_ref (value->data[0].v_pointer) : NULL;
 }
-
-#define __G_PARAM_C__
-#include "gobjectaliasdef.c"
